$include:'globals'
        program P8CONV

c conversion utility for P8 storm files
c formats supported
c  1 = P8 V 1.0 Event-Based Precipitation
c  2 = Earth Info daily format
c  3 = NOAA Release B / Condensed or BASIC Sequential Format
c
c Output format for precip file is:
c  YYYY MM DD P1 P2 .... P24 free format
c
c Output format for air temperature file is:
c  YYYY MM DD Tavg  (deg F)

	character*64 title,ifile,ofile
	integer*4 rd

c input & output units
	data nin/1/,nout /2/

c clear screen
	call clears(-1)
	call locata(0,0)	

 2	write(*,*) 
	write(*,*) 'Utility for Converting P8 Precip Files'
	write(*,*)
	write(*,*) 'Returns File in "YYYY MM DD P1 P2 ... P24" Format'
	write(*,*) '  where Pi = Hourly Precip in Hundreths of Inch'
	write(*,*)
	write(*,*) 'Enter Input File Name or Blank Line to Quit'
	write(*,7) 
	read(*,1) ifile
 1      	format(a64)
	if(len_trim(ifile).le.0) stop
	open(nin,file=ifile,status='old',err=999)

	write(*,*)
	write(*,*) 'Enter Input File Format Type:'
	write(*,*) '1 = P8 Version 1 Storm File'
	write(*,*) '2 = Earth Info. Format File'
	write(*,*) '3 = NOAA Release B/Condensed or Basic Sequential',
     & ' Format'
	write(*,7)
	read(*,*) itype

c turn on trace if type < 0
	if(itype.lt.0) then
		itrace=1
		itype=-itype
		else
		itrace=0
		endif
	
	if(itype.lt.1.or.itype.gt.3) stop

	write(*,*)
	write(*,*) 'Enter Output File Name:'
	write(*,7)
	read(*,1) ofile
	if(len_trim(ofile).le.0) stop
	write(*,*)

	open(nout,file=ofile,err=999)

	write(*,*) 'Enter Output File Description:'

	write(*,7) 
 7        format(' ? '\)
	read(*,1) title
	write(*,*)
	write(*,*)
	write(nout,1) title	

	call rsave(nout,rd,rd,0)
	if(itype.le.1) then
c p8 storm file
			call p8stm(nin,nout,itrace)
		elseif(itype.eq.2) then
c earth info format
			call eif(nin,nout,itrace)
		else
c noaa format
			call noaa(nin,nout,itrace)
		endif
	call rsave(nout,rd,rd,2)
	goto 998

 999    write(*,*) 'File Conversion Error'

 998	close(nout)
	close(nin)
	goto 2 
	end

	subroutine p8stm(nin,nout,itrace)

c convert p8 .stm file format
	integer*4 r,rd(25),idate,id,jd
	integer*2 code
	character*8 cdate
	common /a/ r(5000)

	rd=0
	jd=0

c skip title
	read(nin,*)
 5      code=1
      	read(nin,2,end=999,err=998) iyr,imo,ida,ihr,nwet,nint
 2      	format(4i2,2i4)

	id=idate(iyr,imo,ida)
	if(itrace.gt.0) write(*,*) iyr,imo,ida,ihr,nwet,nint
	if(itrace.gt.0) write(*,*) id,cdate(id)

	if(jd.eq.0) then
			jd=id
			j=ihr-1
		else
			if(jd.ne.id.and.ihr.ne.j.and.iyr.gt.0) then
			write(*,*) 'dates out of sync:'
			write(*,*) cdate(id),ihr
			write(*,*) cdate(jd),j
c				stop
				endif
		endif	
		

	if(nint.gt.5000) then
		write(*,*) 'Storm Duration too Long: ', nint
		stop
		endif
		
	code=2
	do i=1,nint
	        if(i.le.nwet) then
		       	read(1,*,err=999) r(i)
		else
			r(i)=0
		endif
		enddo

	do i=1,nint
		j=j+1
		rd(j)=r(i)
		if(j.eq.24) then
	                call rsave(nout,jd,rd,1)
			rd=0
			j=0
			jd=jd+1
			endif
		enddo
	goto 5

c abnormal ending
 998    continue
	write(*,*) 'File Conversion Error, Code =',code
	stop
c normal ending
 999    continue
	call rsave(nout,jd,rd,1)
	end

	subroutine eif(nin,nout,itrace)
c convert earth info format
c example madison.pre

	character*256 string	
	character*10 date,value
	character*1 c(256)
	equivalence (string,c(1))
	character*8 cdate
	character*1 delim /'|'/
	integer*4 jd,jdl,rd(25),sum,idate

c error code
	integer*2 code

	jdl=0
 30	read(nin,1,end=200,err=999) string
 1       format(a256)

c total length of record
	n=len_trim(string)
	if(n.le.0) goto 30

c skip first 4 fields
	i=0
	j=0
	code=1
 11     i=i+1 
	if(c(i).eq.delim) j=j+1
	if(i.gt.n) goto 999
	if(j.lt.4) goto 11

c read date field
	write(date,2) (c(k),k=i+1,i+10)
 2      	format(10a1)
	code=6
	read(date,6,err=999) im,id,iy
 6  	    format(i2,1x,i2,1x,i4)
	if(itrace.gt.0) write(*,*) date

c read hourly values
	i=i+12
	i1=i
	k=0
 10	if(c(i).eq.delim) then
		code=2
		if((i-1-i1).gt.10) goto 999
		write(value,2) (c(j),j=i1,i-1)
		k=k+1
		read(value,4,err=999) xx
 4              	format(f10.0)
		rd(k) = jfix(xx*100.+.0001)
		if(itrace.gt.0) write(*,*) k,i1,i-1,value,xx,rd(k)
		i=i+1
		code=7
		if(c(i).ne.delim) goto 999
		i1=i+1
		value=' '
		endif
	i=i+1
	if(k.lt.25) goto 10

c check for missing records - fill with zeroes
c	code=3
c	jd=idate(iy,im,id)
c	nmis=jd-jdl-1
c	if(nmis.lt.0) goto 999
c	code=4
c	if(jdl.ne.0.and.nmis.gt.0) then
c		do kk=1,nmis
c		  nfill=nfill+1
c		  jdl=jdl+1
c	 	  call rsave(nout,jdl,rdx,1)
c		  enddo																									
c		endif
	jd=idate(iy,im,id)
	jdl=jd
	code=4
c checksum
	sum=0
	do k=1,24
		sum=sum+rd(k)
		enddo
	if(sum.ne.rd(25)) then	
	     write(*,*) 
	     write(*,*) 'Incorrect Total Precip: ',cdate(jdl),
     &	        sum,rd(25)
	     write(*,*)
	     endif

	call rsave(nout,jd,rd,1)

	goto 30	
				
 999    continue
	write(*,*) 'File Conversion Error, Code =',code
	stop
 200    continue
c	write(*,*)
c	write(*,*) 'Number of Missing Dates (Filled with 0''s) =',
c     &    nfill
	return
	end

	subroutine rsave(nout,id,rd,io)

c save record
	integer*4 rd(1),id,id1,id2
	character*8 char,cdate
	real*4 x(30)

	if(io.eq.0) then
		nrec=0
		nzero=0
	elseif(io.eq.1) then
	sum=0
	do k=1,24
		x(k+3)=rd(k)
		sum=sum+rd(k)
		enddo

c save only if positive precip
	if(id1.eq.0) id1=id
	id2=id
	if(sum.eq.0) then
		nzero=nzero+1
		else
		nrec=nrec+1
		char=cdate(id)
		write(*,22) char
 22       		format('+  ',A10)
		read(char,1) (x(k),k=1,3)
 1 		      format(f4.0,f2.0,f2.0)
		call ocomp(nout,x,27)
		endif
	else
	nzero2=id2-id1+1-nrec-nzero
	write(*,23) cdate(id1),cdate(id2),nrec,nzero,nzero2
 23     format(/' File Conversion Complete'/
     &   ' Date Range = ',a8,' to ',a8/
     &   ' Days with Positive Precip. = ',i8/
     &   ' Days with Zero Precip. =',i8 /
     &   ' Days Missing from Input File =',i8)
	endif	

	return
	end

       subroutine noaa(nin,nout,itrace)
c translate noaa/ncdc hourly precipitation files
c example boston.prn

	integer*2 year,month,day,code
	integer*4 idate,id,id2,idlast,rd(25),jd

 10    code=1
       read(nin,*,end=11,err=999) i

c date range header type=4
       if(i.eq.4) then
	backspace(nin)
	read(nin,*) i,iy1,im1,iy2,im2	

	code=2
c ending date
          day=mday(iy2,im2)
          id2=idate(iy2,im2,day)

c first day
	  idlast=idate(iy1,im1,1)-1
	  id1=idlast
	
          write(*,101) iy1,im1,iy2,im2
 101      format(
     &     ' starting year =',i5,', month =',i3/
     &     '   ending year =',i5,', month =',i3//)
	if(idlast.le.0.or.id2.le.0) goto 999

c day record type=8
       else if(i.eq.8) then
	backspace(nin)
	read(nin,*) i,year,month,day
        year=year+1000

	if(itrace.gt.0) write(*,*) year,month,day
       
c date sequence
	  code=3
          id=idate(year,month,day)
          if(id.le.0) goto 999

c dump buffer
	  if(idlast.ne.id1) call rsave(nout,idlast,rd,1)
c zero
	  rd=0

c fill with zeroes
	  if(id-idlast.gt.1) then
		do jd=idlast+1,id-1
		    call rsave(nout,jd,rd,1)		
		     enddo
		endif
	   idlast=id

c hourly record type=9 with positive precip
	code=3
 15	  read(nin,*,err=999) i,ihr,k
	  if(ihr.eq.25) goto 16
	  if(i.ne.9) goto 999
	  if(ihr.lt.1.or.ihr.gt.25) then
			goto 999
		 elseif(k.gt.0) then
			rd(ihr)=k
		endif
	   goto 15
 16     continue
	endif

	if(i.ne.10) goto 10

c end of file
 11	call rsave(nout,idlast,rd,1)

c fill in last days
	if(id2-id.gt.0) then
		rd=0	
		do jd=id+1,id2
			call rsave(nout,jd,rd,1)
			enddo
		endif			
	goto 998

c error condition	
 999    write(*,*) 'File Conversion Error, Code =',code

 998	close(nin)
	return
	end
